/******************************************************************************\
 * mdf.c - modern df, give informations about the file system usage in a      *
 *         modern way.                                                        *
 *                                                                            *
 *         cfbcruz@dcc.unicamp.br                                             *
 *         nina@dcc.unicamp.br                                                *
\******************************************************************************/

#include <sys/types.h>
#include <sys/vfs.h>
#include <errno.h>
#include <fcntl.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <math.h>

#include "mdf.h"

#ifdef SOLARIS

#include <sys/stat.h>
#include <sys/mnttab.h>

#else

#include <mntent.h>

#endif

#include <malloc.h>



/* global variables */


int used_types;
char types [MAX_TYPES] [MAX_TYPE_LENGTH]; /* show only the asked types */

int used_localizations;
char *localizations [MAX_LOCALIZATIONS]; /* directories or file names */
long local_id_1 [MAX_LOCALIZATIONS];
long local_id_2 [MAX_LOCALIZATIONS];

int used_keywords;
char *keyword [MAX_KEYWORDS];

enum { BL,
       KB,
       MB,
       GB } size_type;

boolean show_only_local;

boolean show_types;

boolean show_inodes; 

boolean show_undelete;

char *undelete_directory;


/* alloc_mem - perform the alocation of memory, only called by
               the define ALLOC
   output    - a pointer to a cleaned allocated memory */
 
void *alloc_mem (word itens, word size) {
 
   static unsigned long int space_used = 0;
 
   void *pointer;
 
   if ((pointer = calloc (itens, size)) == NULL) {
 
     printf ("Out of space. Space used %ld\n", space_used);
     exit (ERROR);
 
   }
 
   space_used += itens * size;
 
#ifdef DEBUG_SPACE
   printf ("Memory required (%d), total (%ld)\n",
           itens * size, space_used);
#endif
 
   return pointer;
}



/* initialize - clear all the variables */
 
void initialize (void) {

  int i;

  show_inodes = FALSE;

  show_only_local = FALSE;

  show_types = FALSE;

  size_type = BL;

  show_undelete = FALSE;

  used_types = -1;
  
  for (i = 0; i < MAX_TYPES; i++) 
    strcpy (types [i], "");

  used_localizations = -1;

  for (i = 0; i < MAX_LOCALIZATIONS; i++) {
    localizations [i] = NULL;
    local_id_1 [i] = local_id_2 [i] = 0;
  } 

  used_keywords = -1;

  for (i = 0; i < MAX_KEYWORDS; i++) 
    keyword [i] = NULL;

  undelete_directory = getenv (UNDELETE_DEFINE);

}



/* get_next_command_line - return the next command in command line
			   return FALSE at the end.
   input - the number of commands and the array of pointers to the
	   commands strings
   output - a pointer to an allocated memory containing the command */
char *get_next_command_line (int argc, char *argv []) {
  
  static int i = 1;

  if (i < argc) { /* there is another command */

    char *command_ptr = ALLOC (strlen (argv [i]), char);
    
    strcpy (command_ptr, argv [i]);
  
    i++;
    
    return command_ptr;
  }
  
  return NULL;
}



/* parse_command_line - interpret the commands given by the user
   input - the number of commands and the array of pointers to the
	   commands strings */

void parse_command_line (int argc, char *argv []) {

  char *command;

  while ( (command = get_next_command_line (argc, argv)) != NULL) {
  
    if (command [0] == OPTION) { /* verify if the command initiates
                                  with the symbol '-' */
      int each_comm;  

      /* nested commands, ex: -ils */
      for (each_comm = 1; each_comm < strlen (command); each_comm++) { 

#ifdef DEBUG_OPTIONS
        printf ("loop option %d\n", each_comm);
#endif

        switch (command [each_comm]) { /* verify which option */
   
          case (INODES): {

            show_inodes = TRUE;

#ifdef DEBUG_OPTIONS
            printf ("Show the inodes\n"); 
#endif
            break;
          }

          case (SPECIFIC_TYPE): {
 
            char *fs_type = get_next_command_line (argc, argv);

            if (fs_type == NULL) {
              fprintf (stderr, "Type not defined\n"); exit (ERROR);
            }         

            if (used_types == MAX_TYPES) {
              fprintf (stderr, "Error: Too many specific types\n");
	      exit (ERROR);
            }

            strcpy (types [++used_types], fs_type);

#ifdef DEBUG_OPTIONS
            printf ("Specific type: %s\n", fs_type);
#endif          
          }
 
          case (KBYTES): {

            size_type = KB;

#ifdef DEBUG_OPTIONS
            printf ("Kbytes\n");
#endif                     
            break;
          }
 
          case (MBYTES): {

            size_type = MB;

#ifdef DEBUG_OPTIONS
            printf ("Mbytes\n");
#endif                     
            break;
          }
  
          case (GBYTES): {

            size_type = GB;

#ifdef DEBUG_OPTIONS
            printf ("Gbytes\n");
#endif                      
            break;
          }

          case (LOCAL): {

            show_only_local = TRUE;

#ifdef DEBUG_OPTIONS
            printf ("Only local\n");
#endif          
            break;
          }
         
          case (SHOW_TYPES): {

            show_types = TRUE;

#ifdef DEBUG_OPTIONS
            printf ("Show types\n");
#endif          
            break;
          }

          case (UNDELETE): {

            show_undelete = TRUE;


#ifdef DEBUG_OPTIONS
            printf ("Show undelete\n");
#endif          
            break;
          }
  
          case (KEYWORD): {
 
            char *key = get_next_command_line (argc, argv);

            if (key == NULL) {
              fprintf (stderr, "Keyword not defined\n"); exit (ERROR);
            }         
  
            if (used_keywords == MAX_KEYWORDS) {
              fprintf (stderr, "Error: Too many keywords\n"); exit (ERROR);
            }

            keyword [++used_keywords] = ALLOC (strlen (key), char);
            strcpy (keyword [used_keywords], key);

#ifdef DEBUG_OPTIONS
            printf ("Keyword specified: %s\n", key);
#endif   
            free (key);           

            break;
          }
   
          default: {
          
            fprintf (stderr, "Illegal option: %s\n", command);
            fprintf (stderr, "Usage:\n");
            fprintf (stderr, 
            "mdf [-ikmglsu] [-t FSType] [-K Keyword] [directory | file ...]\n");
            free (command);
            exit (ERROR);

          }
 
        }

      }

#ifdef DEBUG_OPTIONS
      printf ("Free: %s\n", command);
#endif   
 
      free (command);
    }
 
    else { /* Localization */

#ifdef SOLARIS

      struct statvfs s_statvfs;

      if (used_localizations == MAX_LOCALIZATIONS) {
        fprintf (stderr, "Error: Too many localizations\n"); exit (ERROR);
      }

      localizations [++used_localizations] = ALLOC (strlen (command), char);
      strcpy (localizations [used_localizations], command);

#ifdef DEBUG_OPTIONS
      printf ("Localization: %s\n", command);
#endif   

      if (statvfs (command, &s_statvfs)) {
        fprintf (stderr, "Couldn't get file system information: %s\n", command);
        exit (ERROR);
      }
         
      local_id_1 [used_localizations] = s_statvfs.f_fsid;
      local_id_2 [used_localizations] = s_statvfs.f_fsid; /* redundant value */

      free (command);           

#else

      struct statfs s_statfs;

      if (used_localizations == MAX_LOCALIZATIONS) {
        fprintf (stderr, "Error: Too many localizations\n"); exit (ERROR);
      }

      localizations [++used_localizations] = ALLOC (strlen (command), char);
      strcpy (localizations [used_localizations], command);

#ifdef DEBUG_OPTIONS
      printf ("Localization: %s\n", command);
#endif   

      if (statfs (command, &s_statfs)) {
        fprintf (stderr, "Couldn't get file system information: %s\n", command);
        exit (ERROR);
      }
         
      local_id_1 [used_localizations] = s_statfs.f_fsid.val [0];
      local_id_2 [used_localizations] = s_statfs.f_fsid.val [1];

      free (command);           

#endif
    }

  }

}


#ifdef SOLARIS

file_system *construct_file_system_list (void) {

FILE *file_ptr; /* mounted file system description file pointer */

struct mnttab s_mnttab;

struct statvfs s_statvfs;

file_system *file_system_list = NULL,
            *file_system_ptr = NULL,
	    *file_system_ptr_aux = NULL;

  /* open file for reading */
  file_ptr = fopen (MOUNT_TAB, "r"); 

  /* get all the mounted file systems */
  while (!getmntent (file_ptr, &s_mnttab)) {

    /* if it is a valid type */
    if (TRUE) { 
 
       if (statvfs (s_mnttab.mnt_mountp, &s_statvfs)) {
         fprintf (stderr, "Couldn't get information of %s\n",
	          s_mnttab.mnt_mountp); exit (ERROR);
       }	  
             
       file_system_ptr = ALLOC (1, file_system);
 
       file_system_ptr->mounted_file_name =
         ALLOC (strlen (s_mnttab.mnt_special), char);
       strcpy (file_system_ptr->mounted_file_name, 
               s_mnttab.mnt_special);
 
       file_system_ptr->file_system_path =
         ALLOC (strlen (s_mnttab.mnt_mountp), char);
       strcpy (file_system_ptr->file_system_path, 
               s_mnttab.mnt_mountp);

       file_system_ptr->file_system_id [0] = s_statvfs.f_fsid;         
       file_system_ptr->file_system_id [1] = s_statvfs.f_fsid;         

       file_system_ptr->total_blocks = s_statvfs.f_blocks;

       file_system_ptr->used_blocks = s_statvfs.f_blocks - s_statvfs.f_bfree;

       file_system_ptr->avail_blocks = s_statvfs.f_bavail;

       if (file_system_ptr->avail_blocks < 0)
         file_system_ptr->avail_blocks = 0;

       file_system_ptr->block_size = s_statvfs.f_frsize;

       file_system_ptr->mount_type =
         ALLOC (strlen (s_mnttab.mnt_fstype), char);
       strcpy (file_system_ptr->mount_type, 
               s_mnttab.mnt_fstype);

       file_system_ptr->used_file_nodes = 
         s_statvfs.f_files - s_statvfs.f_ffree;
  
       file_system_ptr->free_file_nodes =
         s_statvfs.f_ffree; 

       file_system_ptr->valid_file_system = TRUE;

       /* insert node into list */
       if (file_system_list == NULL) {
         file_system_list = file_system_ptr;
	 file_system_ptr->next = NULL;
	 file_system_ptr_aux = file_system_ptr;
       }
       else {
         file_system_ptr_aux->next = file_system_ptr;
	 file_system_ptr_aux = file_system_ptr;
	 file_system_ptr->next = NULL;
       }
    }
  }

  fclose (file_ptr);

  return file_system_list;
}

#else

file_system *construct_file_system_list (void) {

FILE *file_ptr; /* mounted file system description file pointer */

struct mntent *s_mntent;

struct statfs s_statfs;

file_system *file_system_list = NULL,
            *file_system_ptr = NULL,
	    *file_system_ptr_aux = NULL;

  /* open file for reading */
  file_ptr = setmntent (MOUNT_TAB, "r"); 

  /* get all the mounted file systems */
  while ( (s_mntent = getmntent (file_ptr)) != NULL ) {

    /* if it is a valid type */
    if (strcmp (s_mntent->mnt_type, TYPE_IGNORE)) { 
 
       if (statfs (s_mntent->mnt_dir, &s_statfs)) {
         fprintf (stderr, "Couldn't get information of %s\n",
	          s_mntent->mnt_dir); exit (ERROR);
       }	  
             
       file_system_ptr = ALLOC (1, file_system);
 
       file_system_ptr->mounted_file_name =
         ALLOC (strlen (s_mntent->mnt_fsname), char);
       strcpy (file_system_ptr->mounted_file_name, 
               s_mntent->mnt_fsname);
 
       file_system_ptr->file_system_path =
         ALLOC (strlen (s_mntent->mnt_dir), char);
       strcpy (file_system_ptr->file_system_path, 
               s_mntent->mnt_dir);

       file_system_ptr->file_system_id [0] = s_statfs.f_fsid.val [0];         
       file_system_ptr->file_system_id [1] = s_statfs.f_fsid.val [1];         

       file_system_ptr->total_blocks = s_statfs.f_blocks;

       file_system_ptr->used_blocks = s_statfs.f_blocks - s_statfs.f_bfree;

       file_system_ptr->avail_blocks = s_statfs.f_bavail;

       if (file_system_ptr->avail_blocks < 0)
         file_system_ptr->avail_blocks = 0;

       file_system_ptr->block_size = s_statfs.f_bsize;

       file_system_ptr->mount_type =
         ALLOC (strlen (s_mntent->mnt_type), char);
       strcpy (file_system_ptr->mount_type, 
               s_mntent->mnt_type);

       file_system_ptr->used_file_nodes = 
         s_statfs.f_files - s_statfs.f_ffree;
  
       file_system_ptr->free_file_nodes =
         s_statfs.f_ffree; 

       file_system_ptr->valid_file_system = TRUE;

       /* insert node into list */
       if (file_system_list == NULL) {
         file_system_list = file_system_ptr;
	 file_system_ptr->next = NULL;
	 file_system_ptr_aux = file_system_ptr;
       }
       else {
         file_system_ptr_aux->next = file_system_ptr;
	 file_system_ptr_aux = file_system_ptr;
	 file_system_ptr->next = NULL;
       }
    }
  }

  endmntent (file_ptr);

  return file_system_list;
}
#endif



/* filter_list - choose the file systems which obey the required options
   input - a pointer to the list of file systems */

void filter_list (file_system *file_system_list) {

  file_system *file_system_ptr = file_system_list;

  while (file_system_ptr != NULL) {

    /* verify if the file system is local */

    if (show_only_local) 
      if (!file_system_ptr->used_file_nodes) 
        file_system_ptr->valid_file_system = FALSE;  

    /* verify types */

    if (used_types != -1) {

      int counter;
      boolean valid_fs;

      valid_fs = FALSE;
      for (counter = 0; counter <= used_types; counter++) 
        if (!strcmp (file_system_ptr->mount_type, types [counter]))
          valid_fs = TRUE;

      if (!valid_fs)
        file_system_ptr->valid_file_system = FALSE;
    }

    /* verify localization by file system identification */

    if (used_localizations != -1) {

      int counter;
      boolean valid_fs;

      valid_fs = FALSE;
      for (counter = 0; counter <= used_localizations; counter++) 
        if (file_system_ptr->file_system_id [0] == local_id_1 [counter] &&
            file_system_ptr->file_system_id [1] == local_id_2 [counter])
          valid_fs = TRUE;

      if (!valid_fs)
        file_system_ptr->valid_file_system = FALSE;
    }

    /* verify file system by the given keywords */

    if (used_keywords != -1) {

      int counter;
      boolean valid_fs;

      valid_fs = FALSE;
      for (counter = 0; counter <= used_keywords; counter++) 
        if (strstr (file_system_ptr->mounted_file_name,
            keyword [counter]) != NULL ||
            strstr (file_system_ptr->file_system_path,
            keyword [counter]) != NULL ||
            strstr (file_system_ptr->mount_type,
            keyword [counter]) != NULL)
          valid_fs = TRUE;

      if (!valid_fs)
        file_system_ptr->valid_file_system = FALSE;
    }

    file_system_ptr = file_system_ptr->next;

  }
 
}



/* show_file_system_list - goes through the list printing the valid 
			   file systems.
   input - a pointer to the list of file systems */

void show_file_system_list (file_system *file_system_list) {

  file_system *file_system_ptr = file_system_list;

  /* print the header */
  if (show_inodes) {

    printf ("   File System            iUsed   iFree %%iUsed "); 

    if (show_types)

      printf ("Type ");
  }  
  else {

    printf ("      File System         "); 

    switch (size_type) {

      case (BL): {

        printf ("Blocks  BL-Used BL-Avail %%Used ");

        break;
      }

      case (KB): {
     
        printf (" KB-Tot KB-Used KB-Avail %%Used ");
       
	break;
      }

      case (MB): {
       
        printf ("MB-Tot  MB-Used  MB-Avail %%Used ");

        break;
      }

      case (GB): {

        printf ("GB-Tot GB-Used GB-Avail %%Used ");

        break;
      }      
    }

    if (show_types)

      printf ("Type ");
  }

  printf ("Mount Point\n");
     
  /* print the file systems */
  while (file_system_ptr != NULL)  {

    if (file_system_ptr->valid_file_system) {

      if (strlen (file_system_ptr->mounted_file_name) > 25) { 
        /* the informations will be printed on the next line */
        printf ("%-25s\n", file_system_ptr->mounted_file_name);

	printf ("%25s", " ");

      }
      else
        /* the informations will be printed on the same line */
        printf ("%-25s", file_system_ptr->mounted_file_name);

      if (show_inodes) { /* inodes */

        if (file_system_ptr->used_file_nodes) {

          float percent = ((float) file_system_ptr->used_file_nodes /
	                  ((float) file_system_ptr->free_file_nodes +
		           (float) file_system_ptr->used_file_nodes)) * 100.0; 


          printf ("%6d  %6d %3d%%",
		   file_system_ptr->used_file_nodes,
		   file_system_ptr->free_file_nodes,
		   (int) percent); 
        }
	else

	   printf ("    *      *      *"); 

        if (show_types)

           printf (" %s ", file_system_ptr->mount_type);

        printf (" %s \n", file_system_ptr->file_system_path);

        }  
        else {  /* file system space */

	  double total_bytes,
	         used_bytes,
	         avail_bytes;

          total_bytes = (double) file_system_ptr->total_blocks * 
		        (double) file_system_ptr->block_size;

          used_bytes = (double) file_system_ptr->used_blocks * 
		       (double) file_system_ptr->block_size;

          avail_bytes = (double) file_system_ptr->avail_blocks *
		         (double) file_system_ptr->block_size;

          switch (size_type) {

            case (BL): {
          
	      float percent_fs = 100.0 - 
				  (((float) file_system_ptr->used_blocks +
				  (float) file_system_ptr->avail_blocks) /
				  (float) file_system_ptr->total_blocks *
				   100.0);

	      float percent = ((float) file_system_ptr->used_blocks /
	                       (float) file_system_ptr->total_blocks) *
			       100; 

              percent += percent_fs;

              printf ("%8d %8d %8d %3d%%",
		      file_system_ptr->total_blocks,
		      file_system_ptr->used_blocks,
		      file_system_ptr->avail_blocks,
		      (int) percent); 

            break;
            }

            case (KB): {

	      float percent_fs = 100.0 - (((float) used_bytes +
				  (float) avail_bytes) /
				  (float) total_bytes * 100.0);

	      float percent = (float) used_bytes
			      / (float) total_bytes * 100.0; 

              percent += percent_fs;

              printf ("%8d %8d %8d %3d%%",
		      (int) (total_bytes / 1024.0),
		      (int) (used_bytes / 1024.0),
		      (int) (avail_bytes / 1024.0),
		      (int) percent);

              break;
            }

            case (MB): {

	      float percent_fs = 100.0 - (((float) used_bytes +
				  (float) avail_bytes) /
				  (float) total_bytes * 100.0);

	      float percent = (float) used_bytes
			      / (float) total_bytes * 100.0; 


              char value [20];
	      char correct_value [20];
	      int count;

              percent += percent_fs;

              /* put all the float pointers vertically */ 
	      sprintf (value, "%6.1f", 
		      (float) ((float) total_bytes / (1024.0 *1024.0)));

              strcpy (correct_value, "");
	      for (count = strlen (value); count < 8; count++)
		strcat (correct_value, " ");
              strcat (correct_value, value);
	      printf ("%s ", correct_value);

	      sprintf (value, "%6.1f", 
		      (float) ((float) used_bytes / (1024.0 * 1024.0)));

              strcpy (correct_value, "");
	      for (count = strlen (value); count < 8; count++)
		strcat (correct_value, " ");
              strcat (correct_value, value);
	      printf ("%s ", correct_value);

	      sprintf (value, "%6.1f", 
		      (float) (float) avail_bytes  /
			       (1024.0 * 1024.0));

              strcpy (correct_value, "");
	      for (count = strlen (value); count < 8; count++)
		strcat (correct_value, " ");
              strcat (correct_value, value);
	      printf ("%s ", correct_value);

              printf ("%3d%%", (int) percent);

              break;
            }

            case (GB): {

	      float percent_fs = 100.0 - (((float) used_bytes +
				  (float) avail_bytes) /
				  (float) total_bytes * 100.0);

	      float percent = (float) used_bytes
			      / (float) total_bytes * 100.0; 

              percent += percent_fs;

              printf ("  %#2.2f    %#2.2f    %#2.2f %3d%%",
		      (float) ((float) total_bytes / (1024.0 *1024.0 * 1024.0)),
		      (float) ((float) used_bytes / (1024.0 * 1024.0 * 1024.0)),
		      (float) avail_bytes /
			       (1024.0 * 1024.0 * 1024.0),
		      (int) percent);

              break;

            }      
          }

          if (show_types)

            printf (" %s", file_system_ptr->mount_type);

          printf (" %s \n", file_system_ptr->file_system_path);
        }
    }

    file_system_ptr = file_system_ptr->next;
  }

}


void show_fs_undelete_list (file_system *file_system_list) {

  file_system *file_system_ptr = file_system_list;

  if (undelete_directory == NULL) {
    fprintf (stderr,"Error: ud_dir not defined (undelete directory)\n");
    fprintf (stderr,"       use setenv ud_dir <directory of undelete>\n");
    exit (ERROR);
  }

  printf ("    File System     "); 

  switch (size_type) {

    case (BL): {

      printf ("Blocks  BL-Used BL-Avail BL_Purge %%Purge %%Used ");

      break;
    }

    case (KB): {
     
      printf (" KB-Tot KB-Used KB-Avail KB_Purge %%Purge %%Used ");
       
      break;
    }

    case (MB): {
       
      printf ("MB-Tot  MB-Used  MB-Avail MB_Purge %%Purge %%Used ");

      break;
    }

    case (GB): {

      printf ("GB-Tot GB-Used GB-Avail GB_Purge %%Purge %%Used ");

      break;
    }      
  }

  if (show_types)

    printf ("Type ");

  printf ("Mount Point\n");
     
  /* print the file systems */
  while (file_system_ptr != NULL)  {

    if (file_system_ptr->valid_file_system) {

      double total_bytes,
             used_bytes,
             avail_bytes;

      char shell_command [1000];

      char mdf_tmp_arq [1000];
      char pid_mdf [10];

      char host_name [1000];
      char *mount_dir;

      float perc, purge;

      boolean infor;

      FILE *mdf_tmp;

      total_bytes = (double) file_system_ptr->total_blocks * 
                    (double) file_system_ptr->block_size;

      used_bytes = (double) file_system_ptr->used_blocks * 
                   (double) file_system_ptr->block_size;

      avail_bytes = (double) file_system_ptr->avail_blocks *
                    (double) file_system_ptr->block_size;

      strcpy (mdf_tmp_arq, "/tmp/mdf");
      sprintf (pid_mdf, "%d", getpid()); 
      strcat (mdf_tmp_arq, pid_mdf);

      if (!file_system_ptr->used_file_nodes) {
	/* remote file system */
	int count;

	count = 0; 
	while ( file_system_ptr->mounted_file_name [count] != '\0' &&
	        file_system_ptr->mounted_file_name [count] != ':')
	count++;

        if (file_system_ptr->mounted_file_name [count] == ':') {

          strcpy (host_name, file_system_ptr->mounted_file_name);
          mount_dir = &host_name [count + 1];
	  host_name [count]=0;

          strcpy (shell_command, "rsh ");
	  strcat (shell_command, host_name);
	  strcat (shell_command, " \"");
          strcat (shell_command, undelete_directory);
          strcat (shell_command, "/");
          strcat (shell_command, "ud_query -v ");
          strcat (shell_command, mount_dir);
	  strcat (shell_command, "\"");
          strcat (shell_command, " | grep Space | cut -c 25- > ");
          strcat (shell_command, mdf_tmp_arq);  

          system (shell_command);

          if ((mdf_tmp = fopen (mdf_tmp_arq,"r")) == NULL) {
            fprintf (stderr, "Erro nao consegui abrir arquivo: %s\n", mdf_tmp_arq);
            exit (ERROR);
          }

          infor = TRUE;
          if (fgetc (mdf_tmp) == EOF)
            infor = FALSE;
          else {
            fseek (mdf_tmp, 0L, 0);

	    fscanf (mdf_tmp, "%f%% (%f)", &perc, &purge);
          }

          fclose (mdf_tmp);

          remove (mdf_tmp_arq); 
	}
	else 
	  infor=FALSE;
	  
      }
      else { /* Local */
        infor = TRUE;

        strcpy (shell_command, "");
        strcat (shell_command, undelete_directory);
        strcat (shell_command, "/");
        strcat (shell_command, "ud_query -v ");
        strcat (shell_command, file_system_ptr->file_system_path);
        strcat (shell_command, " | grep Space | cut -c 25- > ");
        strcat (shell_command, mdf_tmp_arq); 

        system (shell_command);
      
        if ((mdf_tmp = fopen (mdf_tmp_arq,"r")) == NULL) {
          fprintf (stderr, "Erro nao consegui abrir arquivo: %s\n", mdf_tmp_arq);
          exit (ERROR);
        }

        fscanf (mdf_tmp, "%f%% (%f)", &perc, &purge);

        fclose (mdf_tmp);

        remove (mdf_tmp_arq);
      } 
      if (strlen (file_system_ptr->mounted_file_name) > 18) { 
        /* the informations will be printed on the next line */
        printf ("%-18s\n", file_system_ptr->mounted_file_name);

	printf ("%18s", " ");
      }
      else
        /* the informations will be printed on the same line */
        printf ("%-18s", file_system_ptr->mounted_file_name);

      switch (size_type) {

        case (BL): {
          
          float percent_fs = 100.0 - 
			  (((float) file_system_ptr->used_blocks +
  			  (float) file_system_ptr->avail_blocks) /
			  (float) file_system_ptr->total_blocks *
			   100.0);

          float percent = ((float) file_system_ptr->used_blocks /
                        (float) file_system_ptr->total_blocks) *
  		        100; 

          float purge_perc = (((purge * 1024 * 1024) /
			      file_system_ptr->block_size) /
                        (float) file_system_ptr->total_blocks) *
  		        100.0; 

          percent += percent_fs;

          if (infor)
            printf ("%8d %8d %8d %8d %3d%% %3d%%",
 	        file_system_ptr->total_blocks,
	        file_system_ptr->used_blocks,
	        file_system_ptr->avail_blocks,
	        (int) ((purge *1024 * 1024) / file_system_ptr->block_size),
		(int) purge_perc,
	        (int) percent); 
          else
            printf ("%8d %8d %8d    --     --  %3d%%",
 	        file_system_ptr->total_blocks,
	        file_system_ptr->used_blocks,
	        file_system_ptr->avail_blocks,
	        (int) percent); 

          break;
        }

        case (KB): {

          float percent_fs = 100.0 - (((float) used_bytes +
			  (float) avail_bytes) /
			  (float) total_bytes * 100.0);

          float percent = (float) used_bytes
		      / (float) total_bytes * 100.0; 

          float purge_perc = (((purge * 1024 * 1024) /
			      file_system_ptr->block_size) /
                        (float) file_system_ptr->total_blocks) *
  		        100.0; 

          percent += percent_fs;

          if (infor)
            printf ("%8d %8d %8d %8d %3d%% %3d%%",
               (int) (total_bytes / 1024.0),
	       (int) (used_bytes / 1024.0),
	       (int) (avail_bytes / 1024.0),
	       (int) (purge *1024),
	       (int) purge_perc,
	       (int) percent);
          else
            printf ("%8d %8d %8d    --     --  %3d%%",
               (int) (total_bytes / 1024.0),
	       (int) (used_bytes / 1024.0),
	       (int) (avail_bytes / 1024.0),
	       (int) percent);

          break;
        }

        case (MB): {

          float percent_fs = 100.0 - (((float) used_bytes +
			  (float) avail_bytes) /
			  (float) total_bytes * 100.0);

          float percent = (float) used_bytes
 		      / (float) total_bytes * 100.0; 

          float purge_perc = (((purge * 1024 * 1024) /
			      file_system_ptr->block_size) /
                        (float) file_system_ptr->total_blocks) *
  		        100.0; 

          char value [20];
          char correct_value [20];
          int count;

          percent += percent_fs;
  
          /* put all the float pointers vertically */ 
          sprintf (value, "%6.1f", 
                  (float) ((float) total_bytes / (1024.0 *1024.0)));

          strcpy (correct_value, "");
  	  for (count = strlen (value); count < 8; count++)
            strcat (correct_value, " ");
          strcat (correct_value, value);
	  printf ("%s ", correct_value);

          sprintf (value, "%6.1f", 
  	        (float) ((float) used_bytes / (1024.0 * 1024.0)));

          strcpy (correct_value, "");
	  for (count = strlen (value); count < 8; count++)
	  strcat (correct_value, " ");
          strcat (correct_value, value);
  	  printf ("%s ", correct_value);

	  sprintf (value, "%6.1f", 
	          (float) (float) avail_bytes  /
	          (1024.0 * 1024.0));

          strcpy (correct_value, "");
	  for (count = strlen (value); count < 8; count++)
	    strcat (correct_value, " ");
          strcat (correct_value, value);
	  printf ("%s ", correct_value);

	  sprintf (value, "%6.1f", purge);

          if (infor) {
            strcpy (correct_value, "");
	    for (count = strlen (value); count < 8; count++)
	    strcat (correct_value, " ");
            strcat (correct_value, value);
  	    printf ("%s ", correct_value);

            printf ("%3d%% ", (int) purge_perc);
          }
	  else 
	    printf ("   --      -- ");

          printf ("%3d%%", (int) percent);

          break;
        }

        case (GB): {

          float percent_fs = 100.0 - (((float) used_bytes +
	  		    (float) avail_bytes) /
			    (float) total_bytes * 100.0);

          float percent = (float) used_bytes
		      / (float) total_bytes * 100.0; 

          float purge_perc = (((purge * 1024 * 1024) /
			      file_system_ptr->block_size) /
                        (float) file_system_ptr->total_blocks) *
  		        100.0; 

          percent += percent_fs;

          if (infor)
            printf ("  %#2.2f    %#2.2f    %#2.2f      %#2.3f  %3d%%  %3d%%",
	         (float) ((float) total_bytes / (1024.0 *1024.0 * 1024.0)),
	         (float) ((float) used_bytes / (1024.0 * 1024.0 * 1024.0)),
	         (float) avail_bytes /
	         (1024.0 * 1024.0 * 1024.0),
	         (float) (purge / 1024),
		 (int) purge_perc,
	         (int) percent);
          else
            printf ("  %#2.2f    %#2.2f    %#2.2f        --     --   %3d%%",
	         (float) ((float) total_bytes / (1024.0 *1024.0 * 1024.0)),
	         (float) ((float) used_bytes / (1024.0 * 1024.0 * 1024.0)),
	         (float) avail_bytes /
	         (1024.0 * 1024.0 * 1024.0),
	         (int) percent);

          break;

        }      
      }

      if (show_types)

        printf (" %s", file_system_ptr->mount_type);

        printf (" %s \n", file_system_ptr->file_system_path);
    }

    file_system_ptr = file_system_ptr->next;
  }
}

/* free_list - release the memory used by the list of file systems.
   input - a pointer to the file system list */

void free_list (file_system *file_system_list) {

  while (file_system_list != NULL) {
   
    file_system *file_system_ptr = file_system_list;

#ifdef DEBUG_LIST
   
     printf (" %s ", file_system_ptr->mounted_file_name);

     printf (" %s ", file_system_ptr->file_system_path);
    
     printf (" %s \n", file_system_ptr->mount_type);

#endif
    
     free (file_system_ptr->mounted_file_name);

     free (file_system_ptr->file_system_path);
    
     free (file_system_ptr->mount_type);

     file_system_list = file_system_list->next;
 
     free (file_system_ptr);
  }
}



/* mdf - modern df */

void main (int argc, char *argv []) {

  file_system *file_system_list; 

  initialize ();

  parse_command_line (argc, argv);

  file_system_list = construct_file_system_list ();

  filter_list (file_system_list);  

  if (!show_undelete)
    show_file_system_list (file_system_list);
  else
    show_fs_undelete_list (file_system_list);

  free_list (file_system_list);
}

